/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.shell;

import java.awt.Component;
import javax.swing.JProgressBar;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.shell.GImportProgressMonitor;
import org.greenstone.gatherer.shell.GShellProgressMonitor;
/** This implementation of <i>GShellProgressMonitor</i> is designed to parse and translate the progress of a buildcol.pl call.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1
 */
public class GBuildProgressMonitor
    implements GShellProgressMonitor {
    /** Indicates if the GUI has asked the process this object monitors to stop. */
    private boolean stop = false;

    private GImportProgressMonitor import_progress = null;

    private int num_docs = -1;

    private int num_indexes = -1;

    private int state = 0;

    /** The progress bar this monitor updates. */
    private JProgressBar progress_bar;

    static final private int MIN = 0;
    static final private int MAX = 1000000;

    static final int BASE        =  0;
    static final int CTCT        =  1; // Creating the compressed text
    static final int CTCT_CTS    =  2; // Creating the compressed text - collecting text statistics
    static final int CTCT_CTS_PS =  3; // Creating the compressed text - collecting text statistics - parsed successfully
    static final int CTCT_CTCD   =  4; // Creating the compressed text - creating the compression dictionary
    static final int CTCT_CTT    =  5; // Creating the compressed text - compressing the text
    static final int CTCT_CTT_PS =  6; // Creating the compressed text - compressing the text - parsed successfully
    static final int BI          =  7; // Building index
    static final int BI_CID      =  8; // Building index - creating index dictionary
    static final int BI_CID_PS   =  9; // Building index - creating index dictionary - parsed successfully
    static final int BI_ITT      = 10; // Building index - inverting the text
    static final int BI_ITT_PS   = 11; // Building index - inverting the text - parsed successfully
    static final int BI_CTWF     = 12; // Building index - create the weights file
    static final int BI_CODSD    = 13; // Building index - creating 'on-disk' stemmed dictionary
    static final int BI_CSI      = 14; // Building index - creating stem indexes
    static final int CTID        = 15; // Creating the info database
    static final int CTID_PS     = 16; // Creating the info database - parsed successfully
    static final int CAF         = 17; // Creating auxiliary files

    static final String BI_STR   = "*** building index";
    static final String CTS_STR  = "collecting text statistics";
    static final String CTT_STR  = "compressing the text";
    static final String CTWF_STR = "create the weights file";
    static final String CAF_STR = "creating auxiliary files";
    static final String CID_STR  = "creating index dictionary";
    static final String CSI_STR  = "creating stem indexes";
    static final String CTCT_STR = "*** creating the compressed text";
    static final String CTCD_STR = "creating the compression dictionary";
    static final String CTID_STR = "*** creating the info database and processing associated files";
    static final String CODSD_STR = "creating 'on-disk' stemmed dictionary";
    static final String ITT_STR  = "inverting the text";
    static final String PS_STR   = "hash";

    public GBuildProgressMonitor(GImportProgressMonitor import_progress) {
	this.import_progress = import_progress;
	progress_bar = new JProgressBar();
	progress_bar.setMaximum(MAX);
	progress_bar.setMinimum(MIN);
	progress_bar.setValue(MIN);
    }

    /** Method to register a new progress bar with this monitor.
     * @param progress_bar The new <strong>JProgressBar</strong>.
     */	 
    public void addProgressBar(JProgressBar progress_bar) {
	this.progress_bar = progress_bar;
	progress_bar.setMaximum(MAX);
	progress_bar.setMinimum(MIN);
	progress_bar.setValue(MIN);
    }

    /** Determine the script exit value according to the progress monitor. This gets around a problem where several script failures actually result with a successful exit value.
     * @return A <i>int</i> with a value of zero if and only if the script was successful.
     */
    public int exitValue() {
	if(state == BASE) {
	    return 0;
	}
	return 1;
    }
    /** Method to retrieve whatever control is being used as the progress indicator. Usually a <strong>JProgressBar</strong> but there may be others implemented later.
     * @return A <strong>Component</strong> on which the progress of the process is being displayed.
     */
    public Component getProgress() {
	return progress_bar;
    }
    /** Method to determine the state of the stop flag, which may be set by the visual component that created this monitor.
     * @return A <strong>boolean</strong> indicating if the process has been asked to stop.
     */
    public boolean hasSignalledStop() {
	return stop;
    }
    /** Inform the progress bar that it should programatically increment progress by one step. */
    public void increment() {
	// There is no reason this should be called for a build.
    }

    /** This method is used to 'feed in' a line of text captured from the process.
     * @param line A <strong>String</strong> of text captured from either standard out or standard error.
     */
    public void parse(String raw_line) {
	String line = raw_line.toLowerCase();
	// We first check if we can parse, which depends on num_docs > 0 and num_indexes > 0.
	if(num_docs > 0 && num_indexes > 0) {
	    switch(state) {
	    case BASE:
		// We are watching for the beginning of a phase
		if(line.indexOf(CTCT_STR) != -1) {
		    // No progress really.
		    state = CTCT;
		    ///ystem.err.println("CTCT [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		else if(line.indexOf(BI_STR) != -1) {
		    // Skip the progress up to this point.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / (3 + num_indexes)));
		    state = BI;
		    ///ystem.err.println("BI [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		else if(line.indexOf(CTID_STR) != -1) {
		    // Skip the progress up to this point
		    progress_bar.setValue(progress_bar.getValue() + (((1 + num_indexes ) * MAX) / (3 + num_indexes)));
		    state = CTID;
		    ///ystem.err.println("CTID [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		else if(line.indexOf(CAF_STR) != -1) {
		    // Skip the progres up to this point
		    progress_bar.setValue(progress_bar.getValue() + (((2 + num_indexes ) * MAX) / (3 + num_indexes)));
		    state = CAF;
		    ///ystem.err.println("CAF [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CTCT:
		if(line.indexOf(CTS_STR) != -1) {
		    // No progress really.
		    state = CTCT_CTS;
		    ///ystem.err.println("CTS [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CTCT_CTS:
		if(line.indexOf(PS_STR) != -1) {
		    // We have finally found something we can increment because of.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 3 * num_docs)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 3 * num_docs)));
		}
		else if(line.indexOf(CTCD_STR) != -1) {
		    state = CTCT_CTCD;
		    ///ystem.err.println("CTCD [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CTCT_CTCD:
		if(line.indexOf(CTT_STR) != -1) {
		    // Finished creating the compression dictionary.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 3)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 3)));
		    state = CTCT_CTT;
		    ///ystem.err.println("CTT [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CTCT_CTT:
		if(line.indexOf(PS_STR) != -1) {
		    // We have finally found something we can increment because of.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 3 * num_docs)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 3 * num_docs)));
		}
		else if(line.indexOf(BI_STR) != -1) {
		    state = BI;
		    ///ystem.err.println("BI [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI:
		if(line.indexOf(CID_STR) != -1) {
		    state = BI_CID;
		    ///ystem.err.println("CID [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI_CID:
		if(line.indexOf(PS_STR) != -1) {
		    // We have finally found something we can increment because of.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5 * num_docs)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5 * num_docs)));
		}
		else if(line.indexOf(ITT_STR) != -1) {
		    state = BI_ITT;
		    ///ystem.err.println("ITT [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI_ITT:
		if(line.indexOf(PS_STR) != -1) {
		    // We have finally found something we can increment because of.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5 * num_docs)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5 * num_docs)));
		}
		else if(line.indexOf(CTWF_STR) != -1) {
		    state = BI_CTWF;
		    ///ystem.err.println("CTWF [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI_CTWF:
		if(line.indexOf(CODSD_STR) != -1) {
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5)));
		    state = BI_CODSD;
		    ///ystem.err.println("CODSD [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI_CODSD:
		if(line.indexOf(CSI_STR) != -1) {
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5)));
		    state = BI_CSI;
		    ///ystem.err.println("CSI [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case BI_CSI:
		if(line.indexOf(BI_STR) != -1) {
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5)));
		    state = BI;
		    ///ystem.err.println("BI [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		else if(line.indexOf(CTID_STR) != -1) {
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * 5)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * 5)));
		    state = CTID;
		    ///ystem.err.println("CTID [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CTID:
		if(line.indexOf(PS_STR) != -1) {
		    // We have finally found something we can increment because of.
		    progress_bar.setValue(progress_bar.getValue() + (MAX / ((3 + num_indexes) * num_docs)));
		    ///ystem.err.println("Increase " + (MAX / ((3 + num_indexes) * num_docs)));
		}
		else if(line.indexOf(CAF_STR) != -1) {
		    state = CAF;
		    ///ystem.err.println("CAF [" + state + "] - " + ((progress_bar.getValue() * 100) / MAX) + "%");
		}
		break;
	    case CAF:
		progress_bar.setValue(MAX);
		state = BASE;
		///ystem.err.println("Done! 100%");
		break;
	    }
	}
	else {
	    ///ystem.err.println("Number of documents = " + num_docs);
	    ///ystem.err.println("Number of indexes = " + num_indexes);
	    progress_bar.setIndeterminate(true);
	}
    }

    public void reset() {
	state = BASE;
	progress_bar.setValue(MIN);
	num_docs = -1;
	num_indexes = -1;
	stop = false;
	progress_bar.setIndeterminate(false);
    }

    public void saving() {
    }

    /** Since the creator of this process monitor is actually in the GUI, this class provides the simpliest way to send a cancel process message between the two.
     * @param state The desired state of the stop flag as a <strong>boolean</strong>.
     */
    public void setStop(boolean state) {
	stop = state;
	progress_bar.setIndeterminate(false);
    }
    /** This method resets this monitor to the start, reseting the process parsing and progress bar.
     * TODO Everthing.
     */
    public void start() {
	progress_bar.setValue(MIN);
	num_indexes = Gatherer.c_man.getCollection().cdm.index_manager.getSize();
	num_docs = import_progress.getNumberOfDocuments();
    }
    /** This method indicates the process is complete. 
     * TODO Everthing.
     */
    public void stop() {
	progress_bar.setIndeterminate(false);
	progress_bar.setValue(MAX); // Not max, or else it won't blank properly for the next build.
    }
}
